function [U]=HHWeuCallII_MCA(param)
T = 10; 
kappa= 0.5; 
eta=0.04; 
sig1=1; 
sig2=0.09;
rho12=-0.9;
rho13=0;
rho23=0;
a = 0.08;
b=0.10;
S0 = [75 100 125];
V0=0.04;
r0=0.1;
K=100;
N=500;
Ntraj=1e5;
fun=@Sim_Heston_HW;
tic
[tmp(1,:),e0]=fun(T,N,Ntraj,S0,V0,r0,K,[kappa eta sig1 sig2 rho12 rho13 rho23 a b]);
dt=toc;
nrep=ceil((2*max(e0)/param)^2);
if nrep>1
  pool = gcp('nocreate');
  if ~isempty(pool)
   disp(['Estimated running time: '  num2str(dt*(nrep-1)/pool.NumWorkers) ' s.']);
  else
   disp(['Estimated running time: '  num2str(dt*(nrep-1)) ' s.']);   
  end    
end 
parfor i=2:nrep
    %i
 [tmp(i,:)]=fun(T,N,Ntraj,S0,V0,r0,K,[kappa eta sig1 sig2 rho12 rho13 rho23 a b]);
end
U=mean(tmp,1);


function [U,sdev]=Sim_Heston_HW(T,N,Ntraj,S0,v0,r0,K,parameter)

                    % T length of time interval 
                    % N nbr of sample points
                    % Ntraj number of trajectories

kappa=parameter(1);     % volatility mean reversion
eta=parameter(2);    % volatility mean
sig1=parameter(3);    % volatility vol diffusion
sig2=parameter(4);    % volatility r diffusion
rho12=parameter(5);   % diffusion correlation s vol
rho13=parameter(6);  % diffusion correlation s r
rho23=parameter(7);  % diffusion correlation v r
a=parameter(8);     % mean reversion rate r
b=parameter(9);     % mean reversion level r
B=@(t)(1-exp(-a*(T-t)))/a;
dB=@(t) -exp(-a*(T-t));
A=@(t) (B(t)-(T-t))*(b-sig2^2/2/a^2)-B(t)^2*sig2^2/4/a;
P0=exp(-r0*B(0)+A(0));
rho=(rho13-rho12*rho23)/sqrt(1-rho12^2)/sqrt(1-rho23^2);
dt=T/N;
t=0;
d=4*eta*kappa/sig1^2;
lambda=4*kappa*exp(-kappa*dt)/(sig1^2*(1-exp(-kappa*dt)));
konstant=sig1^2*(1-exp(-kappa*dt))/(4*kappa);
V=zeros(Ntraj,1);
X1=V;
X2=V;
IV=V;
V(:,1)=v0;
X1(:,1)=log(1/P0);
X2(:,1)=log(1/P0);
for i=2:(N+1)
    Vnew=konstant*ncx2rnd(d,lambda*V);
    IV=dt*(V+Vnew)/2;
    IsqrtVB=dt*(sqrt(V)*B(t)+sqrt(Vnew)*B(t+dt))/2;
    IB2=dt*(B(t)^2+B(t+dt)^2)/2;
    IdBsqrtV=dt*(sqrt(V)*dB(t)+sqrt(Vnew)*dB(t+dt))/2;
    IBrsqrtV=dt*(B(t)./sqrt(V)+B(t+dt)./sqrt(Vnew))/2;
    IsqrtVdW=(Vnew-V-kappa*eta*dt+kappa*IV)/sig1;
    IBdW=[(IsqrtVB.*IsqrtVdW)./IV+randn(Ntraj,1).*sqrt(max(IB2-IsqrtVB.^2./IV,0))];
    %IBdW=2*(sqrt(Vnew)*B(t+dt)-sqrt(V)*B(t)-IdBsqrtV-IBrsqrtV*(kappa*eta/2-sig1^2/8)+IsqrtVB/2*kappa)/sig1;
    G1=randn(Ntraj,1);
    G2=randn(Ntraj,1);
    X1=X1-0.5*(IV+sig2^2*IB2+2*rho13*IsqrtVB*sig2)+rho12*IsqrtVdW+sig2*rho23*IBdW...
         +sqrt((1-rho12^2)*IV+2*rho*sqrt(1-rho12^2)*sqrt(1-rho23^2)*sig2*IsqrtVB+sig2^2*(1-rho23^2)*rho^2*IB2).*G1...
         +sig2*sqrt(IB2)*sqrt(1-rho23^2)*sqrt(1-rho^2).*G2;
    X2=X2-0.5*(IV+sig2^2*IB2+2*rho13*IsqrtVB*sig2)+rho12*IsqrtVdW+sig2*rho23*IBdW...
         -sqrt((1-rho12^2)*IV+2*rho*sqrt(1-rho12^2)*sqrt(1-rho23^2)*sig2*IsqrtVB+sig2^2*(1-rho23^2)*rho^2*IB2).*G1...
         -sig2*sqrt(IB2)*sqrt(1-rho23^2)*sqrt(1-rho^2).*G2;
    V=Vnew;
    t=t+dt;
    
end 
S1=exp(X1)*S0;
S2=exp(X2)*S0;
SS0=repmat(S0,Ntraj,1);
a=sum(P0/2*(max(S1-K,0)+max(S2-K,0)).*repmat((P0*(exp(X1)+exp(X2))/2-1),1,size(S0,2)))./(sum((P0*(exp(X1)+exp(X2))/2-1).^2));
U=P0*mean((max(S1-K,0)+max(S2-K,0))/2)-mean((P0*(exp(X1)+exp(X2))/2-1))*a;
sdev=P0*std((max(S1-K,0)+max(S2-K,0))/2-(P0*(exp(X1)+exp(X2))/2-1)*a)/sqrt(Ntraj);

function [U,sdev]=Sim_Heston_HW_E(T,N,Ntraj,S0,v0,r0,K,parameter)

                    % T length of time interval 
                    % N nbr of sample points
                    % Ntraj number of trajectories

kappa=parameter(1);     % volatility mean reversion
eta=parameter(2);    % volatility mean
sig1=parameter(3);    % volatility vol diffusion
sig2=parameter(4);    % volatility r diffusion
rho12=parameter(5);   % diffusion correlation s vol
rho13=parameter(6);  % diffusion correlation s r
rho23=parameter(7);  % diffusion correlation v r
a=parameter(8);     % mean=P0*fourieropt(@Heston_HWKf,IS,IS,log(K)*IS,log(S0/P0),0*IS,T*IS,[V0,kappa,eta,sig1,rho12,sig2,a,b],x,we) reversion rate r
b=parameter(9);     % mean reversion level r
B=@(t)(1-exp(-a*(T-t)))/a;
dB=@(t) -exp(-a*(T-t));
A=@(t) (B(t)-(T-t))*(b-sig2^2/2/a^2)-B(t)^2*sig2^2/4/a;
P0=exp(-r0*B(0)+A(0));
rho=(rho13-rho12*rho23)/sqrt(1-rho12^2)/sqrt(1-rho23^2);
dt=T/N;
t=0;
V1=zeros(Ntraj,1);
X1=V1;
V1(:,1)=v0;
V2=V1;
X1(:,1)=log(1/P0);
X2=X1;
rho=[1 rho12 rho13 ; rho12 1 rho23 ; rho13 rho23 1];
A=chol(rho);
for i=2:(N+1)
    dW=randn(Ntraj,3)*A*sqrt(dt);
    X1=X1-1/2*(V1+sig2^2*B(t)^2+2*rho13*sig2*B(t)*sqrt(V1))*dt+sqrt(V1).*dW(:,1)+B(t)*sig2*dW(:,3);
    X2=X2-1/2*(V2+sig2^2*B(t)^2+2*rho13*sig2*B(t)*sqrt(V2))*dt-sqrt(V2).*dW(:,1)-B(t)*sig2*dW(:,3);
    %V1=max(V1+kappa*(eta-V1)*dt+sqrt(V1)*sig1.*dW(:,2)+0*sig1^2/4*(dW(:,2).^2-dt),0);
    %V2=max(V2+kappa*(eta-V2)*dt-sqrt(V2)*sig1.*dW(:,2)+0*sig1^2/4*(dW(:,2).^2-dt),0);
    V1=abs(V1+kappa*(eta-V1)*dt+sqrt(V1)*sig1.*dW(:,2)+sig1^2/4*(dW(:,2).^2-dt));
    V2=abs(V2+kappa*(eta-V2)*dt-sqrt(V2)*sig1.*dW(:,2)+sig1^2/4*(dW(:,2).^2-dt));
    t=t+dt;
end 
S1=exp(X1)*S0;
S2=exp(X2)*S0;
a=sum(P0/2*(max(S1-K,0)+max(S2-K,0)).*repmat((P0*(exp(X1)+exp(X2))/2-1),1,size(S0,2)))./(sum((P0*(exp(X1)+exp(X2))/2-1).^2));
U=P0*mean((max(S1-K,0)+max(S2-K,0))/2)-mean((P0*(exp(X1)+exp(X2))/2-1))*a;
sdev=P0*std((max(S1-K,0)+max(S2-K,0))/2-(P0*(exp(X1)+exp(X2))/2-1)*a)/sqrt(Ntraj);

