function [U1,U2,U3]=QLSVdnTouch_MCA(param)
S=[75 100 125];
V=0.114;
K=100;
T=1;
r=0;
kappa=2.58;
eta=0.043;
sigma=1;
rho=-0.36;
alpha=0.02;
L=50;
H=150;
HH=2/(alpha*L);
LL=2/(alpha*H);
SS=2./(alpha*S);
N=2000;
Ntraj=1e4;
tic
[tmpDNT(1,:),e0]=Sim_DNT(T,N,Ntraj,SS,[V kappa eta sigma rho],LL,HH);
dt=toc;
nrep=ceil((2*max(e0)/param)^2);
if nrep>1
 disp(['Estimated running time: '  num2str(dt*(nrep-1)) ' s.']);
end 
for i=2:nrep
    %i
 [tmpDNT(i,:)]=Sim_DNT(T,N,Ntraj,SS,[V kappa eta sigma rho],LL,HH);
end
DNT=mean(tmpDNT,1);
U1=DNT(1);
U2=DNT(2);
U3=DNT(3);

function [DNT,sdev]=Sim_DNT(T,N,Ntraj,X0,parameter,L,H)

                    % T length of time interval 
                    % N nbr of sample points
                    % Ntraj number of trajectories
v0=parameter(1);    % Initial volatilitet
k=parameter(2);     % volatility mean reversion
th=parameter(3);    % volatility mean
sv=parameter(4);    % volatility diffusion
rho=parameter(5);   % diffusion correlation

dt=T/N;
d=4*th*k/sv^2;
lambda=4*k*exp(-k*dt)/(sv^2*(1-exp(-k*dt)));
konstant=sv^2*(1-exp(-k*dt))/(4*k);
I1=logical(ones(Ntraj,3));
I2=I1;
V=zeros(Ntraj,1);
X1=repmat(X0,Ntraj,1);
X2=X1;
IV=V;
V(:,1)=v0;
 DH1=(H-X1);
 DH2=(H-X2);
 DL1=(X1-L);
 DL2=(X2-L);
for i=2:(N+1)
    Vnew=konstant*ncx2rnd(d,lambda*V);
    IVds=dt*(V+Vnew)/2;
    IsqrtVdW=(Vnew-V-k*th*dt+k*IVds)/sv;
    G=randn(Ntraj,1);
    a=repmat(IVds/2,1,3);
    b1=repmat(-rho*IsqrtVdW+sqrt((1-rho^2)*IVds).*G,1,3);
    b2=repmat(-rho*IsqrtVdW-sqrt((1-rho^2)*IVds).*G,1,3);
    X1=1/2*(X1+a./X1+b1+sqrt((X1+a./X1+b1).^2+4*a));
    X2=1/2*(X2+a./X2+b2+sqrt((X2+a./X2+b2).^2+4*a));
    U1=repmat(rand(Ntraj,1),1,3);
    U2=repmat(rand(Ntraj,1),1,3);
    P11=exp(-DH1.*(H-X1)./a);
    P12=exp(-DH2.*(H-X2)./a);
    P21=exp(-DL1.*(X1-L)./a);
    P22=exp(-DL2.*(X2-L)./a);
    I1=I1&(X1>L)&(X1<H)&(P11<U1)&(P21<U2);
    I2=I2&(X2>L)&(X2<H)&(P12<U1)&(P22<U2);
    DH1=(H-X1);
    DH2=(H-X2);
    DL1=(X1-L);
    DL2=(X2-L);
    V=Vnew;
end   
DNT=mean((I1+I2)/2);
sdev=std((I1+I2)/2)/sqrt(Ntraj);

