function [U1,U2,U3]=QLSVeuCall_MCA(param)
S=[75 100 125];
V=0.114;
K=100;
T=1;
r=0;
kappa=2.58;
eta=0.043;
sigma=1;
rho=-0.36;
alpha=0.02;
SS=2./(alpha*S);
N=500;
Ntraj=1e5;
tic
[tmpU(1,:),e0]=Sim_QLSV(T,N,Ntraj,SS,K,r,[V kappa eta sigma rho alpha]);
dt=toc;
nrep=ceil((2*max(e0)/param)^2);
if nrep>1
 disp(['Estimated running time: '  num2str(dt*(nrep-1)) ' s.']);
end 
for i=2:nrep
    %i
 [tmpU(i,:)]=Sim_QLSV(T,N,Ntraj,SS,K,r,[V kappa eta sigma rho alpha]);
end
U=mean(tmpU,1);
U1=U(1);
U2=U(2);
U3=U(3);

function [U,sdev]=Sim_QLSV(T,N,Ntraj,X0,K,r,parameter)

                    % T length of time interval 
                    % N nbr of sample points
                    % Ntraj number of trajectories
v0=parameter(1);    % Initial volatilitet
k=parameter(2);     % volatility mean reversion
th=parameter(3);    % volatility mean
sv=parameter(4);    % volatility diffusion
rho=parameter(5);   % diffusion correlation
alpha=parameter(6);

dt=T/N;
d=4*th*k/sv^2;
lambda=4*k*exp(-k*dt)/(sv^2*(1-exp(-k*dt)));
konstant=sv^2*(1-exp(-k*dt))/(4*k);
V=zeros(Ntraj,1);
X1=repmat(X0,Ntraj,1);
X2=X1;
IV=V;
V(:,1)=v0;
for i=2:(N+1)
    Vnew=konstant*ncx2rnd(d,lambda*V);
    IVds=dt*(V+Vnew)/2;
    IsqrtVdW=(Vnew-V-k*th*dt+k*IVds)/sv;
    G=randn(Ntraj,1);
    a=repmat(IVds/2,1,3)-r*dt;
    b1=repmat(-rho*IsqrtVdW+sqrt((1-rho^2)*IVds).*G,1,3);
    b2=repmat(-rho*IsqrtVdW-sqrt((1-rho^2)*IVds).*G,1,3);
    tmp1=X1+a./X1+b1;
    tmp2=X2+a./X2+b2;
    X1=1/2*(tmp1+sqrt(tmp1.*tmp1+4*a));
    X2=1/2*(tmp2+sqrt(tmp2.*tmp2+4*a));
    V=Vnew;
end   
TU1=exp(-r*T)*max(2./(alpha*X1)-K,0);
TU2=exp(-r*T)*max(2./(alpha*X2)-K,0);
sdev=std((TU1+TU2)/2)/sqrt(Ntraj);
U=mean((TU1+TU2)/2);

